<?php

include __DIR__ . '/../vendor/autoload.php';

use Discord\Discord;
use Discord\Parts\Channel\Message;
use Discord\WebSockets\Intents;
use Illuminate\Database\Capsule\Manager as Capsule;
use Madcoda\Youtube\Youtube;
use Noodlehaus\Config;

use Pigeon\Models\Video;

// load config file
$config = new Config('config/config.yaml');

// create Eloquent connection object
$capsule = new Capsule();
$capsule->addConnection([
  'driver'   => 'sqlite',
  'database' => $config->get('pigeon.db_path'),
]);
$capsule->setAsGlobal();
$capsule->bootEloquent();

// create YouTube API object
$youtube = new Youtube(array('key' => $config->get('youtube.api_key')));
// get channel
$yt_res = $youtube->getChannelById($config->get('youtube.channel_id'));
// get channel info
$config->set('youtube.channel_name', $yt_res->snippet->title);
$config->set('youtube.uploads_id', $yt_res->contentDetails->relatedPlaylists->uploads);

// get the channel's uploads so we can seed the DB
$uploads = $youtube->getPlaylistItemsByPlaylistId($config->get('youtube.uploads_id'));

// create Video model objects with the above uploads
foreach ($uploads as $u) {
  $video = new Video();
  $video->video_id = $u->contentDetails->videoId;
  $video->published_at = date($u->contentDetails->videoPublishedAt);

  // create the table entry if it doesn't already exist
  Video::where('video_id', $video->video_id)->firstOr(function () use ($video) {
    $video->save();
  });
}

// create our event loop
$loop = React\EventLoop\Factory::create();

// instantiate Discord API object
$discord = new Discord([
  'token' => $config->get('discord.api_key'),
  'intents' => Intents::getDefaultIntents() | Intents::GUILDS,
  'loop' => $loop,
]);

// periodic youtube uploads check
$loop->addPeriodicTimer($config->get('pigeon.check_interval') * 60, function () use ($config, $discord, $youtube) {
  // get latest video from uploads playlist
  $uploads = $youtube->getPlaylistItemsByPlaylistId($config->get('youtube.uploads_id'), 1);
  $uploadDetails = $uploads[0]->contentDetails;

  // search for the latest video in the DB
  Video::where('video_id', $uploadDetails->videoId)->firstOr(function () use ($discord, $config, $uploadDetails) {
    $latest_url = "https://youtu.be/$uploadDetails->videoId";

    $message_text = str_replace('{link}', $latest_url, $config->get('discord.message_template'));

    $discord->guilds->first()->channels->find(function ($channel) use ($config) {
      return $channel->name === $config->get('discord.announcement_channel');
    })->sendMessage($message_text)->done(function ($message) use ($discord) {
      echo 'Message sent!', PHP_EOL;
    });

    // add the video to the DB
    $video = new Video();
    $video->video_id = $uploadDetails->videoId;
    $video->published_at = date($uploadDetails->videoPublishedAt);
    $video->save();
  });
});

// run tasks when Discord is ready
$discord->on('ready', function (Discord $discord) {
  echo "Bot is ready!", PHP_EOL;
});

$discord->run();
